(* $Id: main.ml,v 1.14 2001/11/16 00:00:46 malekith Exp $ *)

(* Change this to false in production release (if there is any...) *)
let compile_only = ref false
let generate_ksi_only = ref false
let save_temps = ref false
let ksi_compiler = ref "xgcc"
let ksi_opts : string list ref = ref []
let input_files : string list ref = ref []
let verbose_flag = ref false
let output_file = ref "a.out"
let prefix = ref Config.prefix
;;

let verb s = if !verbose_flag then (print_endline s; flush stdout) else ()
let err s = prerr_endline ((Array.get Sys.argv 0) ^ ": " ^ s); flush stderr
;;

let xf = Printf.sprintf
;;

let split_filename s =
  if String.contains s '.' && 
     (not (String.contains s '/') || 
      String.rindex s '.' > String.rindex s '/') then
    let suf_idx = String.rindex s '.' in
      (String.sub s 0 suf_idx, 
       String.sub s (suf_idx + 1) (String.length s - suf_idx - 1))
  else
    (s, "")
;;

let execute argv =
  let cmd = String.concat " " argv in
  verb (xf "executing `%s'" cmd);
  let ret = Sys.command cmd in
  if ret != 0 then
    (err (xf "`%s' failed, exit code = %d" cmd ret);
     exit 1)
  else
    ()
;;

let compile_file f = 
  let compile_ksi i o =
    verb (xf "compiling ksi file `%s' -> `%s'" i o);
    execute ((!ksi_compiler :: !ksi_opts) @ [ "-o"; o; "-c"; i ]) in
  let compile_gont i o =
    verb (xf "compiling gont file `%s' -> `%s'" i o);
    let e = Gont.compile !prefix i o in
    if e != 0 then
       (if (not !save_temps) && (Sys.file_exists o) then
          Sys.remove o
        else
          ();
        exit 1) 
     else 
       () in
  let (basename, suffix) = split_filename f in
    match suffix with
        "g" -> 
          let aux = (basename ^ ".ksi") in
          let out = (basename ^ ".o") in
          compile_gont f aux;
          if !generate_ksi_only then
            aux
          else
            (compile_ksi aux out; if not !save_temps then Sys.remove aux; out)
      | "o" -> f
      | "ksi" ->
          if !generate_ksi_only then
            f
          else
            let out = (basename ^ ".o") in
            (compile_ksi f out; out)
      | _ ->
          err (xf "don't know what to do with `%s'" f);
          exit 1
;;

let link fs =
  let o = !output_file in
  verb (xf "linking `%s' -> `%s'" (String.concat "', `" fs) o);
  execute ((!ksi_compiler :: !ksi_opts) @ 
          [ "-o"; o; ] @ 
          fs @ 
          [!prefix ^ "/stdlib.o"])
;;

open Arg

let main () =
  let add_ksi_opt x = 
     ksi_opts := x :: !ksi_opts in
  let add_input_file x =
     input_files := x :: !input_files in
  let prog = (Array.get Sys.argv 0) in
  let help_str = 
        prog ^ " - compile and link gont files\n\n\
        If input file name matches *.g, it is assumed to be Gont source\n\
        file, is compiled to *.ksi file, which is in turn, passed to Ksi\n\
        compiler. *.ksi files are passed directly to Ksi compiler.\n" in
  let opt = [
    ("-c", Set compile_only, "         just compile");
    ("-save-temps", Set save_temps, "save temporary files");
    ("-ksi", String add_ksi_opt, "OPT    pass option to ksi compiler");
    ("-ksicc", String (fun s -> ksi_compiler := s), "PROG set ksi compiler");
    ("-O0", Unit (fun () -> add_ksi_opt "-O0"), "        same as -ksi -O0");
    ("-O1", Unit (fun () -> add_ksi_opt "-O1"), "        same as -ksi -O1");
    ("-O2", Unit (fun () -> add_ksi_opt "-O2"), "        same as -ksi -O2");
    ("-O3", Unit (fun () -> add_ksi_opt "-O3"), "        same as -ksi -O3");
    ("-g", Unit (fun () -> add_ksi_opt "-g"), "         same as -ksi -g");
    ("-S", Set generate_ksi_only, "         just generate .ksi file and stop");
    ("-o", String (fun s -> output_file := s), "OUTFILE  set linker output file name");
    ("-B", String (fun s -> prefix := s ), "PREFIX   overide prefix (default " 
                                           ^ !prefix ^ ")");
    ("-v", Set verbose_flag, "         be verbose")
  ] in

  Arg.parse opt add_input_file help_str;
  ksi_opts := List.rev !ksi_opts;
  (match !input_files with
      [] -> err "so what?";
            err ("run " ^ prog ^ " -help to get some info");
            exit 1
    | _ -> ());
  let out = List.map compile_file (List.rev !input_files) in
    if !compile_only || !generate_ksi_only then
      ()
    else
      link out;
  exit 0
;;

main ()
;;
